@@ -150,8 +150,14 @@ class Purchase(
             'readonly': ((Eval('state') != 'draft')
                 | (Eval('lines', [0]) & Eval('currency'))),
             })
-    lines = fields.One2Many('purchase.line', 'purchase', 'Lines',
-        states=_states)
+    lines = fields.One2Many(
+        'purchase.line', 'purchase', "Lines",
+        states={
+            'readonly': (
+                (Eval('state') != 'draft')
+                | ~Eval('company')
+                | ~Eval('currency')),
+            })
     comment = fields.Text('Comment')
     untaxed_amount = fields.Function(Monetary(
             "Untaxed", currency='currency', digits='currency'),
@@ -1834,6 +1840,9 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
         if move.on_change_with_unit_price_required():
             move.unit_price = self.unit_price
             move.currency = self.purchase.currency
+        else:
+            move.unit_price = None
+            move.currency = None
         move.planned_date = self.planned_delivery_date
         move.invoice_lines = self._get_move_invoice_lines(move_type)
         move.origin = self
