@@ -2,16 +2,12 @@
 # this repository contains the full copyright notices and license terms.
 from functools import wraps
 
-from sql import Null
-from sql.aggregate import Max
-from sql.operators import Concat
-
 from trytond.i18n import gettext
-from trytond.model import Workflow, ModelView, fields
+from trytond.model import ModelView, Workflow, fields
 from trytond.model.exceptions import AccessError
-from trytond.pyson import Eval
 from trytond.pool import Pool, PoolMeta
-from trytond.transaction import Transaction
+from trytond.pyson import Eval
+from trytond.transaction import Transaction, without_check_access
 
 
 def process_purchase(moves_field):
@@ -20,12 +16,16 @@ def process_purchase(moves_field):
         def wrapper(cls, shipments):
             pool = Pool()
             Purchase = pool.get('purchase.purchase')
-            with Transaction().set_context(_check_access=False):
+            transaction = Transaction()
+            context = transaction.context
+            with without_check_access():
                 purchases = set(m.purchase for s in cls.browse(shipments)
                     for m in getattr(s, moves_field) if m.purchase)
             func(cls, shipments)
             if purchases:
-                Purchase.__queue__.process(purchases)
+                with transaction.set_context(
+                        queue_batch=context.get('queue_batch', True)):
+                    Purchase.__queue__.process(purchases)
         return wrapper
     return _process_purchase
 
@@ -46,8 +46,7 @@ class ShipmentIn(metaclass=PoolMeta):
                 add_remove,
                 cls.incoming_moves.add_remove,
                 ]
-        if 'supplier' not in cls.incoming_moves.depends:
-            cls.incoming_moves.depends.append('supplier')
+        cls.incoming_moves.depends.add('supplier')
 
     @classmethod
     @ModelView.button
@@ -69,6 +68,12 @@ class ShipmentIn(metaclass=PoolMeta):
     @Workflow.transition('received')
     @process_purchase('incoming_moves')
     def receive(cls, shipments):
+        pool = Pool()
+        PurchaseLine = pool.get('purchase.line')
+        for shipment in shipments:
+            for move in shipment.incoming_moves:
+                if isinstance(move.origin, PurchaseLine):
+                    move.origin.check_move_quantity()
         super(ShipmentIn, cls).receive(shipments)
 
     @classmethod
@@ -82,35 +87,6 @@ class ShipmentIn(metaclass=PoolMeta):
 class ShipmentInReturn(metaclass=PoolMeta):
     __name__ = 'stock.shipment.in.return'
 
-    @classmethod
-    def __register__(cls, module_name):
-        pool = Pool()
-        Move = pool.get('stock.move')
-        PurchaseLine = pool.get('purchase.line')
-        Purchase = pool.get('purchase.purchase')
-        cursor = Transaction().connection.cursor()
-        sql_table = cls.__table__()
-        move = Move.__table__()
-        line = PurchaseLine.__table__()
-        purchase = Purchase.__table__()
-
-        # Migration from 3.8: New supplier field
-        cursor.execute(*sql_table.select(sql_table.supplier,
-                where=sql_table.supplier == Null, limit=1))
-        if cursor.fetchone():
-            value = sql_table.join(move, condition=(
-                    Concat(cls.__name__ + ',', sql_table.id) == move.shipment)
-                ).join(line, condition=(
-                        Concat(PurchaseLine.__name__ + ',', line.id)
-                        == move.origin)
-                ).join(purchase,
-                    condition=(purchase.id == line.purchase)
-                ).select(Max(purchase.party))
-            cursor.execute(*sql_table.update(
-                    columns=[sql_table.supplier],
-                    values=[value]))
-        super(ShipmentInReturn, cls).__register__(module_name)
-
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
@@ -134,18 +110,26 @@ class ShipmentInReturn(metaclass=PoolMeta):
         super(ShipmentInReturn, cls).done(shipments)
 
 
-def process_purchase_move(func):
-    @wraps(func)
-    def wrapper(cls, moves):
-        pool = Pool()
-        Purchase = pool.get('purchase.purchase')
-        with Transaction().set_context(_check_access=False):
-            purchases = set(
-                m.purchase for m in cls.browse(moves) if m.purchase)
-        func(cls, moves)
-        if purchases:
-            Purchase.__queue__.process(purchases)
-    return wrapper
+def process_purchase_move(without_shipment=False):
+    def _process_purchase_move(func):
+        @wraps(func)
+        def wrapper(cls, moves):
+            pool = Pool()
+            Purchase = pool.get('purchase.purchase')
+            transaction = Transaction()
+            context = transaction.context
+            with without_check_access():
+                p_moves = cls.browse(moves)
+                if without_shipment:
+                    p_moves = [m for m in p_moves if not m.shipment]
+                purchases = set(m.purchase for m in p_moves if m.purchase)
+            func(cls, moves)
+            if purchases:
+                with transaction.set_context(
+                        queue_batch=context.get('queue_batch', True)):
+                    Purchase.__queue__.process(purchases)
+        return wrapper
+    return _process_purchase_move
 
 
 class Move(metaclass=PoolMeta):
@@ -158,7 +142,7 @@ class Move(metaclass=PoolMeta):
             context={
                 'company': Eval('company', -1),
                 },
-            depends=['company']),
+            depends={'company'}),
         'get_supplier', searcher='search_supplier')
     purchase_exception_state = fields.Function(fields.Selection([
         ('', ''),
@@ -213,7 +197,7 @@ class Move(metaclass=PoolMeta):
         # Use getattr as reference field can have negative id
         if (isinstance(self.origin, PurchaseLine)
                 and getattr(self.origin, 'unit', None)):
-            category = self.origin.unit.category.id
+            category = self.origin.unit.category
         return category
 
     @property
@@ -227,18 +211,25 @@ class Move(metaclass=PoolMeta):
 
     @classmethod
     def search_supplier(cls, name, clause):
-        return [('origin.purchase.party' + clause[0].lstrip(name),)
-            + tuple(clause[1:3]) + ('purchase.line',) + tuple(clause[3:])]
+        return [('origin.purchase.party' + clause[0][len(name):],
+                *clause[1:3], 'purchase.line', *clause[3:])]
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('done')
+    @process_purchase_move(without_shipment=True)
+    def do(cls, moves):
+        super().do(moves)
 
     @classmethod
     @ModelView.button
     @Workflow.transition('cancelled')
-    @process_purchase_move
+    @process_purchase_move(without_shipment=True)
     def cancel(cls, moves):
         super(Move, cls).cancel(moves)
 
     @classmethod
-    @process_purchase_move
+    @process_purchase_move()
     def delete(cls, moves):
         super(Move, cls).delete(moves)
 
@@ -255,5 +246,4 @@ class Location(metaclass=PoolMeta):
             ('type', '=', 'storage'),
             ('parent', 'child_of', [Eval('id', -1)]),
             ],
-        depends=['type', 'id'],
         help='If empty the Storage location is used.')
